@@ -1,4 +1,4 @@
-/* $Id: alglin1.c,v 1.32.2.3 2001/09/28 11:48:46 cvsadmin Exp $
+/* $Id: alglin1.c,v 1.32.2.4 2002/10/05 14:53:13 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1117,27 +1117,26 @@ keri(GEN x)
     else
     {
       GEN p0 = p;
-      long av1;
 
       c[j]=k; d[k]=j; p = gcoeff(x,j,k);
 
       for (t=1; t<=m; t++)
 	if (t!=j)
 	{
-	  GEN q=gcoeff(x,t,k), p1,p2;
+	  GEN q=gcoeff(x,t,k), p1;
 	  for (i=k+1; i<=n; i++)
 	  {
-	    av1=avma; (void)new_chunk(lgefint(p0));
-	    p1=mulii(q,gcoeff(x,j,i));
-	    p2=mulii(p,gcoeff(x,t,i));
-	    p1=subii(p2,p1); avma=av1;
-	    coeff(x,t,i) = ldivii(p1,p0);
+	    ulong av1 = avma;
+	    p1 = subii(mulii(p,gcoeff(x,t,i)), mulii(q,gcoeff(x,j,i)));
+	    coeff(x,t,i) = (long)gerepileuptoint(av1, divii(p1,p0));
 	  }
 	  if (low_stack(lim, stack_lim(av,1)))
           {
-            p1 = gclone(p);
+            GEN _p0 = gclone(p0);
+            GEN _p  = gclone(p);
             gerepile_gauss_ker(x,m,n,k,t,av);
-            p = gcopy(p1); gunclone(p1);
+            p = icopy(_p);  gunclone(_p);
+            p0= icopy(_p0); gunclone(_p0);
           }
 	}
     }
