@@ -1,4 +1,4 @@
-/* $Id: base2.c,v 1.38.2.6 2001/06/27 12:00:53 karim Exp $
+/* $Id: base2.c,v 1.38.2.8 2002/05/15 17:03:17 bill Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1277,7 +1277,7 @@ nilord(GEN p, GEN fx, long mf, GEN gx, long flag)
 
   for(;;)
   {
-    /* kappa need to be recomputed */
+    /* kappa needs to be recomputed */
     kapp = NULL;
     Fa   = degree(nu);
     /* the prime element in Zp[alpha] */
@@ -1331,6 +1331,7 @@ nilord(GEN p, GEN fx, long mf, GEN gx, long flag)
       if (DEBUGLEVEL >= 5)
 	fprintferr("  beta = %Z\n", beta);
 
+      /* if pmf divides norm(beta) then it's useless */
       p1 = gmod(gnorm(gmodulcp(beta, chi)), pmf);
       if (signe(p1))
       {
@@ -1380,20 +1381,22 @@ nilord(GEN p, GEN fx, long mf, GEN gx, long flag)
 
       if (!gcmp1(denom(content(chig))))
       {
-	/* the valuation of beta was wrong... This also means 
-           that chi_gamma has more than one factor modulo p    */
-	vb = vstar(p, chig);
-	eq = (long)(-vb[0] / vb[1]);
-	er = (long)(-vb[0]*Ea / vb[1] - eq*Ea);
-	if (eq) gamm = gmul(gamm, gpowgs(p, eq));
-	if (er) 
-        {
-	  gamm = gmul(gamm, gpowgs(nu, er));
+	/* Valuation of beta was wrong. This means that
+	   gamma fails the v*-test */
+	p1   = mulii(pdr, ggcd(denom(content(beta)), pdr));
+	chib = mycaract(redelt(chi, mulii(pdr, p1), pdr), beta);
+	vb = vstar(p, chib);
+	eq = (long)(vb[0] / vb[1]);
+	er = (long)(vb[0]*Ea / vb[1] - eq*Ea);
+ 	if (eq) gamm = gdiv(beta, gpowgs(p, eq));
+	else gamm = beta;
+	if (er)
+	{
+	  gamm = gdiv(gamm, gpowgs(nu, er));
 	  gamm = gmod(gamm, chi);
 	  gamm = redelt(gamm, p, pmr);
 	}
-	p1   = mulii(pdr, ggcd(denom(content(gamm)), pdr));
-	chig = mycaract(redelt(chi, mulii(pdr, p1), pdr), gamm);
+	chig = mycaract(chi, gamm);
       }
 
       chig = polmodi(chig, pmr);
@@ -1437,6 +1440,8 @@ nilord(GEN p, GEN fx, long mf, GEN gx, long flag)
 
       for (i = 1;; i++)
       {
+        if (i >= lg(w))
+          err(talker, "bug in nilord (no suitable root), is p a prime?");
         delt = gneg_i(gsubst(gcoeff(w, 2, i), nv, polx[v]));
         eta  = gsub(gamm, delt);	  
         if (typ(delt) == t_INT)
