@@ -1,4 +1,4 @@
-/* $Id: gen3.c,v 1.22.2.3 2001/09/15 11:17:55 karim Exp $
+/* $Id: gen3.c,v 1.22.2.4 2002/04/06 12:39:10 cvsadmin Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -546,16 +546,32 @@ gdivgs(GEN x, long s)
       z[1]=x[1]; return z;
 
     case t_RFRAC:
-      z = cgetg(3, t_RFRAC);
-      i = ggcd(stoi(s),(GEN)x[1])[2];
-      avma = (long)z;
-      if (i == 1)
+      av = avma;
+      p1 = ggcd(stoi(s),(GEN)x[1]);
+      if (typ(p1) == t_INT)
       {
-        z[2]=lmulsg(s,(GEN)x[2]);
-        z[1]=lcopy((GEN)x[1]); return z;
+        avma = av;
+        z = cgetg(3, t_RFRAC);
+        i = p1[2];
+        if (i == 1)
+        {
+          z[1] = lcopy((GEN)x[1]);
+          z[2] = lmulsg(s,(GEN)x[2]);
+        }
+        else
+        {
+          z[1] = ldivgs((GEN)x[1], i);
+          z[2] = lmulgs((GEN)x[2], s/i);
+        }
       }
-      z[1] = ldivgs((GEN)x[1], i);
-      z[2] = lmulgs((GEN)x[2], s/i); return z;
+      else /* t_FRAC */
+      {
+        z = cgetg(3, t_RFRAC);
+        z[1] = ldiv((GEN)x[1], p1);
+        z[2] = lmul((GEN)x[2], gdivsg(s,p1));
+        z = gerepileupto(av, gcopy(z));
+      }
+      return z;
 
     case t_RFRACN: z=cgetg(3,t_RFRACN);
       z[2]=lmulsg(s,(GEN)x[2]);
@@ -1275,37 +1291,48 @@ gsubst(GEN x, long v, GEN y)
 GEN
 recip(GEN x)
 {
-  long tetpil, av=avma, v=varn(x);
+  long v=varn(x), lx = lg(x);
+  long tetpil, av=avma;
   GEN p1,p2,a,y,u;
 
   if (typ(x)!=t_SER) err(talker,"not a series in serreverse");
-  if (valp(x)!=1) err(talker,"valuation not equal to 1 in serreverse");
+  if (valp(x)!=1 || lx < 3)
+    err(talker,"valuation not equal to 1 in serreverse");
 
   a=(GEN)x[2];
   if (gcmp1(a))
   {
-    long i,j,k, lx=lg(x), lim=stack_lim(av,2);
+    long i, j, k, mi;
+    long lim=stack_lim(av, 2);
 
-    u=cgetg(lx,t_SER); y=cgetg(lx,t_SER);
-    u[1]=y[1]=evalsigne(1) | evalvalp(1) | evalvarn(v);
-    u[2]=un; u[3]=lmulsg(-2,(GEN)x[3]);
-    y[2]=un; y[3]=lneg((GEN)x[3]);
+    mi = lx-1; while (mi>=3 && gcmp0((GEN)x[mi])) mi--;
+    u = cgetg(lx,t_SER);
+    y = cgetg(lx,t_SER);
+    u[1] = y[1] = evalsigne(1) | evalvalp(1) | evalvarn(v);
+    u[2] = y[2] = un;
+    if (lx > 3)
+    {
+      u[3] = lmulsg(-2,(GEN)x[3]);
+      y[3] = lneg((GEN)x[3]);
+    }
     for (i=3; i<lx-1; )
     {
       for (j=3; j<i+1; j++)
       {
-        p1=(GEN)u[j];
-        for (k=j-1; k>2; k--)
-          p1=gsub(p1,gmul((GEN)u[k],(GEN)x[j-k+2]));
-        u[j]=lsub(p1,(GEN)x[j]);
+        p1 = (GEN)x[j];
+        for (k=max(3,j+2-mi); k<j; k++)
+          p1 = gadd(p1, gmul((GEN)u[k],(GEN)x[j-k+2]));
+        u[j] = lsub((GEN)u[j], p1);
       }
-      p1=gmulsg(i,(GEN)x[i+1]);
-      for (k=2; k<i; k++)
+      p1 = gmulsg(i,(GEN)x[i+1]);
+      for (k=2; k<min(i,mi); k++)
       {
-        p2=gmul((GEN)x[k+1],(GEN)u[i-k+2]);
-        p1=gadd(p1,gmulsg(k,p2));
+        p2 = gmul((GEN)x[k+1],(GEN)u[i-k+2]);
+        p1 = gadd(p1, gmulsg(k,p2));
       }
-      i++; u[i]=lneg(p1); y[i]=ldivgs((GEN)u[i],i-1);
+      i++;
+      u[i] = lneg(p1);
+      y[i] = ldivgs((GEN)u[i],i-1);
       if (low_stack(lim, stack_lim(av,2)))
       {
 	GEN *gptr[2];
@@ -1314,11 +1341,11 @@ recip(GEN x)
 	gptr[0]=&u; gptr[1]=&y; gerepilemany(av,gptr,2);
       }
     }
-    tetpil=avma; return gerepile(av,tetpil,gcopy(y));
+    return gerepileupto(av, gcopy(y));
   }
-  y=gdiv(x,a); y[2]=un; y=recip(y);
-  a=gdiv(polx[v],a); tetpil=avma;
-  return gerepile(av,tetpil,gsubst(y,v,a));
+  y = gdiv(x,a); y[2] = un; y = recip(y);
+  a = gdiv(polx[v],a); tetpil = avma;
+  return gerepile(av,tetpil, gsubst(y,v,a));
 }
 
 /*******************************************************************/
