@@ -1,4 +1,4 @@
-/* $Id: trans2.c,v 1.11 2000/11/03 21:00:23 karim Exp $
+/* $Id: trans2.c,v 1.11.2.2 2002/06/01 15:52:04 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -479,7 +479,7 @@ mpch(GEN x)
 GEN
 gch(GEN x, long prec)
 {
-  long av,tetpil;
+  long av;
   GEN p1;
 
   switch(typ(x))
@@ -487,14 +487,11 @@ gch(GEN x, long prec)
     case t_REAL:
       return mpch(x);
 
-    case t_COMPLEX:
-      av=avma; p1=gexp(x,prec);
-      p1=gadd(p1,ginv(p1)); tetpil=avma;
-      return gerepile(av,tetpil,gmul2n(p1,-1));
-
     case t_SER:
-      av=avma; p1=gexp(x,prec); p1=gadd(p1,gdivsg(1,p1));
-      tetpil=avma; return gerepile(av,tetpil,gmul2n(p1,-1));
+      if (gcmp0(x) && valp(x) == 0) return gcopy(x);
+    case t_COMPLEX:
+      av = avma; p1 = gexp(x,prec); p1 = gadd(p1, ginv(p1));
+      return gerepileupto(av, gmul2n(p1,-1));
 
     case t_INTMOD: case t_PADIC:
       err(typeer,"gch");
