@@ -1,4 +1,4 @@
-/* $Id: anal.c,v 1.45.2.4 2001/11/04 14:59:45 cvsadmin Exp $
+/* $Id: anal.c,v 1.45.2.7 2002/06/22 16:34:14 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -35,6 +35,8 @@ typedef struct var_cell {
 
 #define separe(c) ((c)==';' || (c)==':')
 typedef GEN (*PFGEN)(ANYARG);
+typedef GEN (*F2GEN)(GEN,GEN);
+typedef GEN (*F1GEN)(GEN);
 
 static GEN    constante();
 static GEN    expr();
@@ -258,6 +260,10 @@ install(void *f, char *name, char *code)
   if (ep) err(warner,"[install] '%s' already there. Not replaced", name);
   else
   {
+    char *s = name;
+    if (isalpha((int)*s))
+      while (is_keyword_char(*++s)) /* empty */;
+    if (*s) err(talker2,"not a valid identifier", s, name);
     ep = installep(f, name, strlen(name), EpINSTALL, 0, functions_hash + hash);
     ep->code = pari_strdup(code);
   }
@@ -468,105 +474,104 @@ gshift_l(GEN x, GEN n)  { return gshift(x, itos(n)); }
 static GEN
 gshift_r(GEN x, GEN n) { return gshift(x,-itos(n)); }
 
+#define UNDEF (GEN)0x1
 static GEN
 expr(void)
 {
-  PFGEN f[] = { NULL,NULL,NULL,NULL };
+  long av = avma, lim = stack_lim(av, 2);
   GEN aux,e,e1,e2,e3;
-  long av = avma, lim = stack_lim(av,2);
-
-  e1 = e2 = e3 = NULL;
+  F2GEN F1,F2,F3;
+  int F0 = 0;
 
+  F1 = F2 = F3 = (F2GEN)NULL;
+  e1 = e2 = e3 = UNDEF;
 L3:
   aux = facteur();
   if (br_status) return NULL;
-  e3 = f[3]? f[3](e3,aux): aux;
+  e3 = F3? F3(e3,aux): aux;
   switch(*analyseur)
   {
-    case '*': analyseur++; f[3] = (PFGEN)&gmul; goto L3;
-    case '/': analyseur++; f[3] = (PFGEN)&gdiv; goto L3;
-    case '%': analyseur++; f[3] = (PFGEN)&gmod; goto L3;
+    case '*': analyseur++; F3 = &gmul; goto L3;
+    case '/': analyseur++; F3 = &gdiv; goto L3;
+    case '%': analyseur++; F3 = &gmod; goto L3;
     case '\\':
-      if (*++analyseur == '/') { analyseur++; f[3]=(PFGEN)&gdivround; goto L3; }
-      f[3] = (PFGEN)&gdivent; goto L3;
+      if (analyseur[1] != '/') { analyseur++; F3 = &gdivent; goto L3; }
+      analyseur += 2; F3=&gdivround; goto L3;
 
-    case '<': case '>':
-      if (analyseur[1] == *analyseur)
-      {
-        f[3] = (*analyseur == '<')? (PFGEN)&gshift_l
-                                  : (PFGEN)&gshift_r;
-        analyseur += 2; goto L3;
-      }
+    case '<':
+      if (analyseur[1] != '<') break;
+      analyseur += 2; F3 = &gshift_l; goto L3;
+    case '>':
+      if (analyseur[1] != '>') break;
+      analyseur += 2; F3 = &gshift_r; goto L3;
   }
-  f[3] = NULL;
+  F3 = (F2GEN)NULL;
 
 L2:
-  if (!e3) goto L3;
-  e2 = f[2]? f[2](e2,e3): e3;
-  e3 = NULL;
+  if (e3 == UNDEF) goto L3;
+  e2 = F2? F2(e2,e3): e3;
+  e3 = UNDEF;
   if (low_stack(lim, stack_lim(av,2)))
   {
-    GEN *gptr[2];
-    int n = 1; gptr[0]=&e2;
-    if (e1) gptr[n++]=&e1;
+    GEN *gptr[2]; gptr[0]=&e2; gptr[1]=&e1;
     if(DEBUGMEM>1) err(warnmem,"expr");
-    gerepilemany(av,gptr,n);
+    gerepilemany(av,gptr,(e1==UNDEF)?1: 2);
   }
 
   switch(*analyseur)
   {
-    case '+': analyseur++; f[2]=(PFGEN)&gadd; goto L3;
-    case '-': analyseur++; f[2]=(PFGEN)&gsub; goto L3;
+    case '+': analyseur++; F2=&gadd; goto L3;
+    case '-': analyseur++; F2=&gsub; goto L3;
   }
-  f[2] = NULL;
+  F2 = (F2GEN)NULL;
 
 L1:
-  if (!e2) goto L2;
-  e1 = f[1]? f[1](e1,e2): e2;
-  e2 = NULL;
+  if (e2 == UNDEF) goto L2;
+  e1 = F1? F1(e1,e2): e2;
+  e2 = UNDEF;
   switch(*analyseur)
   {
     case '<':
       switch(*++analyseur)
       {
-        case '=': analyseur++; f[1]=(PFGEN)&gle; goto L2;
-        case '>': analyseur++; f[1]=(PFGEN)&gne; goto L2;
+        case '=': analyseur++; F1=&gle; goto L2;
+        case '>': analyseur++; F1=&gne; goto L2;
       }
-      f[1]=(PFGEN)&glt; goto L2;
+      F1=&glt; goto L2;
 
     case '>':
-      if (*++analyseur == '=') { analyseur++; f[1]=(PFGEN)&gge; goto L2; }
-      f[1]=(PFGEN)&ggt; goto L2;
+      if (*++analyseur == '=') { analyseur++; F1=&gge; goto L2; }
+      F1=&ggt; goto L2;
 
     case '=':
-      if (analyseur[1] == '=') { analyseur+=2; f[1]=(PFGEN)&geq; goto L2; }
+      if (analyseur[1] == '=') { analyseur+=2; F1=&geq; goto L2; }
       goto L1;
 
     case '!':
-      if (analyseur[1] == '=') { analyseur+=2; f[1]=(PFGEN)&gne; goto L2; }
+      if (analyseur[1] == '=') { analyseur+=2; F1=&gne; goto L2; }
       goto L1;
   }
-  f[1] = NULL;
+  F1 = (F2GEN)NULL;
 
 /* L0: */
-  if (!e1) goto L1;
-  e = f[0]? (gcmp0(e1)? gzero: gun): e1;
-  e1 = NULL;
+  if (e1 == UNDEF) goto L1;
+  e = F0? (gcmp0(e1)? gzero: gun): e1;
+  e1 = UNDEF;
   switch(*analyseur)
   {
     case '&':
       if (*++analyseur == '&') analyseur++;
       if (gcmp0(e)) { skipexpr(); return gzero; }
-      f[0]=(PFGEN)1L; goto L1;
+      F0=1; goto L1;
 
     case '|':
       if (*++analyseur == '|') analyseur++;
       if (!gcmp0(e)) { skipexpr(); return gun; }
-      f[0]=(PFGEN)1L; goto L1;
+      F0=1; goto L1;
   }
   return e;
 }
-
+#undef UNDEF
 /********************************************************************/
 /**                                                                **/
 /**                        CHECK FUNCTIONS                         **/
