@@ -1,4 +1,4 @@
-/* $Id: compat.c,v 1.7.2.1 2001/03/26 11:56:38 karim Exp $
+/* $Id: compat.c,v 1.7.2.2 2002/06/04 09:35:18 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -43,7 +43,7 @@ rounderror(GEN x)
 }
 
 static void
-suppressed(ANYARG) {err(talker,"this function has been suppressed");}
+suppressed(void) {err(talker,"this function has been suppressed");}
 
 entree oldfonctions[]={
 {"O",50,NULL,7,NULL},
